@@ -1,10 +1,9 @@
 /**
  * jQuery Mobile UI application logic.
  *
- * Copyright 2005-2013 Horde LLC (http://www.horde.org/)
- *
- * See the enclosed file COPYING for license information (GPL). If you
- * did not receive this file, see http://www.horde.org/licenses/gpl.
+ * @author     Michael Slusarz <slusarz@horde.org>
+ * @copyright  2005-2014 Horde LLC
+ * @license    GPL-2 (http://www.horde.org/licenses/gpl)
  */
 var ImpMobile = {
 
@@ -22,6 +21,9 @@ var ImpMobile = {
     // /* The current compose type. */
     // composetype,
     //
+    // /* Flag information. */
+    // flags,
+    //
     // /* Has the folders list been loaded? */
     // foldersLoaded,
     //
@@ -91,7 +93,7 @@ var ImpMobile = {
         case 'compose':
             $('#imp-compose-to-addr,#imp-compose-cc-addr').empty();
             if (!IMP.conf.disable_compose) {
-                $('#imp-compose-cache').val('');
+                $('#imp-compose-cache,#imp-compose-hmac').val('');
                 ImpMobile.compose(data);
             }
             e.preventDefault();
@@ -106,7 +108,7 @@ var ImpMobile = {
         case 'compose-cancel':
         case 'compose-discard':
             HordeMobile.doAction('cancelCompose', {
-                discard: Number(view == 'compose-discard'),
+                discard: ~~(view == 'compose-discard'),
                 imp_compose: $('#imp-compose-cache').val()
             });
             ImpMobile.closeCompose();
@@ -151,8 +153,6 @@ var ImpMobile = {
                 HordeMobile.doAction('poll', {
                     poll: JSON.stringify([])
                 });
-            } else {
-                ImpMobile.loadFolders();
             }
             break;
 
@@ -189,7 +189,7 @@ var ImpMobile = {
             $.mobile.changePage(HordeMobile.createUrl('mailbox', {
                 mbox: ImpMobile.mailbox
             }), {
-                data: { norefresh: true }
+                data: { force: true }
             });
             e.preventDefault();
             break;
@@ -319,7 +319,7 @@ var ImpMobile = {
 
     /**
      */
-    beforeShow: function(e, data)
+    beforeShow: function(undefined, data)
     {
         switch (HordeMobile.currentPage()) {
         case 'compose':
@@ -329,6 +329,22 @@ var ImpMobile = {
             break;
 
         case 'copymove':
+            if (!$('#imp-copymove-list').children().size()) {
+                HordeMobile.doAction(
+                    'copyMoveMailboxList',
+                    {},
+                    function(r) {
+                        $('#imp-copymove-list')
+                            .html(r)
+                            .change(function() {
+                                $.fn[$(this[this.selectedIndex]).hasClass('flistCreate') ? 'show' : 'hide'].call($('#imp-copymove-newdiv'));
+                            })
+                            .find('.flistMailboxes OPTION[value=""]').attr('disabled', true);
+                    },
+                    { async: false }
+                );
+            }
+
             $('#imp-copymove')[0].reset();
             $('#imp-copymove-action,#imp-copymove-list').selectmenu('refresh', true);
             $('#imp-copymove-action').selectmenu(ImpMobile.cache[ImpMobile.mailbox].readonly ? 'disable' : 'enable');
@@ -345,24 +361,21 @@ var ImpMobile = {
 
     /**
      */
-    pageShow: function(e, opts)
+    pageShow: function()
     {
         switch (HordeMobile.currentPage()) {
+        case 'folders':
+            if (!ImpMobile.foldersLoaded) {
+                ImpMobile.loadFolders();
+            }
+            break;
+
         case 'mailbox':
             $.mobile.silentScroll(ImpMobile.mailboxTop);
 
             // Need to do here since Exit Search does not trigger beforeShow.
             $.fn[ImpMobile.search ? 'hide' : 'show'].call($('#imp-mailbox-search'));
             $.fn[ImpMobile.search ? 'show' : 'hide'].call($('#imp-mailbox-searchedit'));
-
-            if (IMP.conf.refresh_time) {
-                window.clearInterval(ImpMobile.refresh);
-                ImpMobile.refresh = window.setInterval(function() {
-                    if (HordeMobile.currentPage() == 'mailbox') {
-                        ImpMobile.refreshMailbox();
-                    }
-                }, IMP.conf.refresh_time * 1000);
-            }
             break;
 
         case 'message':
@@ -380,6 +393,7 @@ var ImpMobile = {
     {
         var purl = data.options.parsedUrl,
             mailbox = purl.params.mbox || ImpMobile.INBOX,
+            noajax = purl.params.noajax || (data.options.data && data.options.data.noajax),
             title = $('#imp-mailbox-' + mailbox).text(),
             params, ob;
 
@@ -417,17 +431,17 @@ var ImpMobile = {
 
         HordeMobile.changePage('mailbox', data);
 
-        if ((!data.options.data || !data.options.data.norefresh) &&
+        if ((!data.options.data || !data.options.data.force) &&
             (ob = ImpMobile.cache[mailbox])) {
             ImpMobile.refreshMailbox(ob);
-            if (data.options.data && data.options.data.noajax) {
+            if (noajax) {
                 return;
             }
             params.checkcache = 1;
         }
 
         HordeMobile.doAction(
-            'viewPort',
+            'viewport',
             ImpMobile.addViewportParams($.extend(params, {
                 view: mailbox
             }))
@@ -440,7 +454,8 @@ var ImpMobile = {
     {
         params = params || {};
 
-        var ob = ImpMobile.cache[ImpMobile.mailbox], slice;
+        var p, slice,
+            ob = ImpMobile.cache[ImpMobile.mailbox];
 
         if (ob) {
             params.cache = ImpMobile.toUidString(ob.cachedIds());
@@ -448,6 +463,7 @@ var ImpMobile = {
             slice = ob.slice;
         } else {
             params.initial = 1;
+            params.force = 1;
             slice = ImpMobile.mbox_slice;
         }
 
@@ -455,10 +471,16 @@ var ImpMobile = {
             params.slice = '1:' + slice;
         }
 
-        return {
+        p = {
             view: params.view,
             viewport: JSON.stringify(params)
         };
+
+        if (!ImpMobile.flags) {
+            p.flag_config = 1;
+        }
+
+        return p;
     },
 
     /**
@@ -497,25 +519,32 @@ var ImpMobile = {
             ob.disappear(r.disappear);
         }
 
-        if (HordeMobile.currentPage() == 'mailbox') {
-            ImpMobile.refreshMailbox(ob);
-        }
+        ImpMobile.resetRefresh();
+        ImpMobile.refreshMailbox(ob);
     },
 
     /**
+     * Refresh the current mailbox.
+     *
      * @param object ob  If ob is not present, triggers an AJAX refresh.
      *                   Otherwise, rebuilds mailbox using ob.
      */
     refreshMailbox: function(ob)
     {
         if (!ob) {
-            HordeMobile.doAction(
-                'viewPort',
-                ImpMobile.addViewportParams({
-                    checkcache: 1,
-                    view: ImpMobile.mailbox
-                })
-            );
+            if (HordeMobile.currentPage() != 'folders') {
+                HordeMobile.doAction(
+                    'viewPort',
+                    ImpMobile.addViewportParams({
+                        checkcache: 1,
+                        view: ImpMobile.mailbox
+                    })
+                );
+            }
+            return;
+        }
+
+        if (HordeMobile.currentPage() != 'mailbox') {
             return;
         }
 
@@ -551,6 +580,8 @@ var ImpMobile = {
         $.each(ob.rows(), function(key, val) {
             var c = $('<li class="imp-message"></li>')
                     .jqmData('buid', val.buid),
+                labels = [],
+                tmp,
                 url = HordeMobile.createUrl('message', {
                     buid: val.buid,
                     mbox: ImpMobile.mailbox
@@ -574,18 +605,43 @@ var ImpMobile = {
                     case IMP.conf.flags.seen:
                         c.addClass('imp-mailbox-seen');
                         break;
+
+                    default:
+                        /* All other flags are added as labels. */
+                        if (ImpMobile.flags[flag]) {
+                            tmp = ImpMobile.flags[flag];
+                            if (tmp.s) {
+                                labels.push(
+                                    $('<span></span>')
+                                        .addClass('imp-mailbox-label')
+                                        .css({
+                                            background: (tmp.b ? tmp.b : ''),
+                                            color: tmp.f
+                                        })
+                                        .text(tmp.l)
+                                );
+                            }
+                        }
+                        break;
                     }
                 });
             }
 
             list.append(
                 c.append(
-                    $('<a href="' + url + '"></a>').html(val.data.subject)).append(
+                    $('<a href="' + url + '"></a>').text(val.data.subject)
+                ).append(
                     $('<div class="imp-mailbox-secondrow"></div>').append(
                         $('<span class="imp-mailbox-date"></span>').text(
-                            val.data.date)).append(
+                            val.data.date
+                        )
+                    ).append(
                         $('<span class="imp-mailbox-from"></span>').text(
-                            val.data.from))));
+                            val.data.from
+                        ).append(labels)
+                    )
+                )
+            );
         });
 
         if (ob.totalrows > ob.slice) {
@@ -597,6 +653,20 @@ var ImpMobile = {
         list.listview('refresh');
     },
 
+    /**
+     * Resets the current mailbox refresh interval.
+     */
+    resetRefresh: function()
+    {
+        if (IMP.conf.refresh_time) {
+            window.clearInterval(ImpMobile.refresh);
+            ImpMobile.refresh = window.setInterval(
+                ImpMobile.refreshMailbox,
+                IMP.conf.refresh_time * 1000
+            );
+        }
+    },
+
     /**
      * Switches to the message view and loads a message.
      *
@@ -627,7 +697,6 @@ var ImpMobile = {
         HordeMobile.doAction(
             'showMessage',
             $.extend(ImpMobile.addViewportParams($.extend(params, {
-                force: 1,
                 view: purl.params.mbox
             })), {
                 buid: purl.params.buid
@@ -684,30 +753,25 @@ var ImpMobile = {
 
         ImpMobile.buid = r.buid;
 
+        document.title = data.title;
         $('#message .smartmobile-title').text(data.title);
-        document.title = $('#message .smartmobile-title').text();
 
         tmp = (ImpMobile.mailbox == IMP.conf.qsearchid);
         $('#message .smartmobile-back').attr(
             'href',
             HordeMobile.createUrl('mailbox', {
-                mbox: (tmp ? IMP.conf.qsearchid : ImpMobile.mailbox)
+                mbox: (tmp ? IMP.conf.qsearchid : ImpMobile.mailbox),
+                noajax: 1
             })
         ).find('.ui-btn-text').text(
             tmp ? IMP.text.searchresults : cache.label
         );
 
-        if (!data.from) {
-            $('#imp-message-from').text(IMP.text.nofrom);
-        } else if (data.from.raw) {
-            $('#imp-message-from').text(data.from.raw);
-        } else if (data.from.addr[0].g) {
-            $('#imp-message-from').text(data.from.addr[0].g);
-        } else if (data.from.addr[0].p) {
-            $('#imp-message-from').text(data.from.addr[0].p);
-        } else {
-            $('#imp-message-from').text(data.from.addr[0].b);
-        }
+        $('#imp-message-from').text(
+            data.from
+                ? (data.from.raw || data.from.addr[0].g || data.from.addr[0].p || data.from.addr[0].b)
+                : IMP.text.nofrom
+        );
 
         if (data.atc_label) {
             $('#imp-message-atc').show();
@@ -751,10 +815,6 @@ var ImpMobile = {
         tmp = $('#imp-message-headers');
         $.mobile.silentScroll(parseInt(tmp.position().top, 10) + parseInt(tmp.height(), 10) - $('#message > :jqmData(role=header)').height());
 
-        $.each($('#imp-message-body IFRAME.htmlMsgData'), function(k, v) {
-            IMP_JS.iframeResize($(v));
-        });
-
         delete ImpMobile.message;
     },
 
@@ -935,7 +995,7 @@ var ImpMobile = {
                 view: ImpMobile.mailbox
             }), {
                 buid: buid,
-                spam: Number(action == 'spam')
+                spam: ~~(action == 'spam')
             })
         );
     },
@@ -948,7 +1008,7 @@ var ImpMobile = {
      */
     compose: function(data)
     {
-        var cache, func,
+        var func,
             params = {},
             purl = data.options.parsedUrl;
 
@@ -957,8 +1017,6 @@ var ImpMobile = {
         if (purl.params.to || purl.params.cc) {
             ImpMobile.addAddress('to', purl.params.to);
             ImpMobile.addAddress('cc', purl.params.cc);
-            HordeMobile.changePage('compose', data);
-            return;
         }
 
         $('#imp-compose-form').show();
@@ -973,13 +1031,11 @@ var ImpMobile = {
         case 'reply_auto':
         case 'reply_list':
             func = 'getReplyData';
-            cache = '#imp-compose-cache';
             params.format = 'text';
             break;
 
         case 'forward_auto':
             func = 'smartmobileGetForwardData';
-            cache = '#imp-compose-cache';
             params.format = 'text';
             break;
 
@@ -987,18 +1043,16 @@ var ImpMobile = {
             $('#imp-compose-form').hide();
             $('#imp-redirect-form').show();
             func = 'getRedirectData';
-            cache = '#imp-redirect-cache';
             break;
 
         case 'resume':
         case 'template':
             func = 'getResumeData';
-            cache = '#imp-compose-cache';
             params.format = 'text';
             break;
 
         default:
-            HordeMobile.changePage('compose');
+            HordeMobile.changePage('compose', data);
             return;
         }
 
@@ -1006,7 +1060,6 @@ var ImpMobile = {
             func,
             $.extend(params, {
                 buid: purl.params.buid,
-                imp_compose: $(cache).val(),
                 type: purl.params.type,
                 view: purl.params.mbox
             }),
@@ -1104,8 +1157,12 @@ var ImpMobile = {
     addAddress: function(f, addr)
     {
         if (addr) {
-            var elt = $('#imp-compose-' + f + '-addr');
-            elt.append(
+            $('#imp-compose-' + f + '-addr').prepend(
+                $('<input></input>')
+                    .attr('name', f + '[]')
+                    .attr('type', 'hidden')
+                    .val(addr)
+            ).prepend(
                 $('<a></a>')
                     .attr('href', '#compose-delete-addr')
                     .attr('data-role', 'button')
@@ -1115,12 +1172,48 @@ var ImpMobile = {
                     .text(addr)
                     .button()
             );
-            elt.append(
-                $('<input></input>')
-                    .attr('name', f + '[]')
-                    .attr('type', 'hidden')
-                    .val(addr)
-            );
+        }
+    },
+
+    /**
+     */
+    processComposeAddress: function(e, prev)
+    {
+        var chr, pos = 0, tmp,
+            val = $(this).val();
+
+        if (val.length > prev.length) {
+            // Optimization when only one character was added
+            if (val.length - prev.length == 1) {
+                tmp = val.charAt(val.length - 1);
+                if (tmp != ',' && tmp != ';') {
+                    return;
+                }
+            }
+
+            chr = val.charAt(pos);
+            while (chr !== "") {
+                switch (chr) {
+                case ',':
+                case ';':
+                    if (!pos) {
+                        val = val.substr(1);
+                    } else if (val.charAt(pos - 1) != '\\') {
+                        ImpMobile.addAddress($(this).attr('id') == 'imp-compose-to' ? 'to' : 'cc', $.trim(val.substr(0, chr == ';' ? pos + 1 : pos)));
+                        val = val.substr(pos + 2);
+                        pos = 0;
+                    }
+                    break;
+
+                default:
+                    ++pos;
+                    break;
+                }
+
+                chr = val.charAt(pos);
+            }
+
+            $(this).val($.trim(val));
         }
     },
 
@@ -1129,12 +1222,14 @@ var ImpMobile = {
      */
     loadFolders: function()
     {
+        ImpMobile.foldersLoaded = false;
+
         HordeMobile.doAction(
             'smartmobileFolderTree',
             { all: ImpMobile.showAllFolders() },
             function(r) {
-                ImpMobile.foldersLoaded = true;
                 $('#imp-folders-list').html(r).listview('refresh');
+                ImpMobile.foldersLoaded = true;
             }
         );
     },
@@ -1168,6 +1263,9 @@ var ImpMobile = {
     closeCompose: function()
     {
         $('#imp-compose-form')[0].reset();
+        $.each($('#imp-compose-to,#imp-compose-cc'), function(undefined, v) {
+            $(v).autocomplete('clear');
+        });
         window.history.back();
     },
 
@@ -1198,7 +1296,7 @@ var ImpMobile = {
 
         if (cmlist.find(':selected').hasClass('flistCreate')) {
             opts.newmbox = $.trim($('#imp-copymove-new').val());
-            if (opts.newmbox == "") {
+            if (opts.newmbox === "") {
                 window.alert(IMP.text.move_nombox);
                 return;
             }
@@ -1210,7 +1308,7 @@ var ImpMobile = {
             move ? 'moveMessages' : 'copyMessages',
             $.extend(ImpMobile.addViewportParams({
                 checkcache: 1,
-                force: Number(move),
+                force: ~~(move),
                 view: source
             }), opts, {
                 buid: $('#imp-copymove-buid').val()
@@ -1253,7 +1351,7 @@ var ImpMobile = {
                                 ob = tmp;
                             }
                             if (v.remove) {
-                                ob = $.grep(ob, function(n, i) {
+                                ob = $.grep(ob, function(n) {
                                     return $.inArray(n, v.remove) < 0;
                                 });
                             }
@@ -1271,10 +1369,10 @@ var ImpMobile = {
      */
     updateFolders: function(r)
     {
-        /* If directly loading the folders page, the poll task will be
-         * executed before this task. */
-        if ((HordeMobile.currentPage() == 'folders') &&
-            !$('#imp-folders-list').children().size()) {
+        /* If redrawing the folders via smartmobileFolderTree action, this
+         * task will be executed before the list is refreshed. */
+        if (!ImpMobile.foldersLoaded &&
+            (HordeMobile.currentPage() == 'folders')) {
             window.setTimeout(function() { ImpMobile.updateFolders(r); }, 100);
         }
 
@@ -1367,7 +1465,12 @@ var ImpMobile = {
         if ((v = d['imp:compose'])) {
             $.fn[v.atclimit ? 'hide' : 'show'].call($('#imp-compose-attach-form'));
             if (v.cacheid) {
-                $($('#imp-redirect-form:visible').length ? '#imp-redirect-cache' : '#imp-compose-cache').val(v.cacheid);
+                if ($('#imp-redirect-form:visible').length) {
+                    $('#imp-redirect-cache').val(v.cacheid);
+                } else {
+                    $('#imp-compose-cache').val(v.cacheid);
+                    $('#imp-compose-hmac').val(v.hmac);
+                }
             }
         }
 
@@ -1381,6 +1484,13 @@ var ImpMobile = {
             ImpMobile.mailboxCache = null;
         }
 
+        if ((v = d['imp:flag-config'])) {
+            ImpMobile.flags = {};
+            $.each(v, function(undefined, value) {
+                ImpMobile.flags[value.id] = value;
+            });
+        }
+
         if ((v = d['imp:message'])) {
             ImpMobile.message = v.shift().data;
         }
@@ -1450,17 +1560,14 @@ var ImpMobile = {
      */
     onDocumentReady: function()
     {
-        $(document).on('pagebeforechange', ImpMobile.toPage)
-            .on('pagebeforeshow', ImpMobile.beforeShow)
-            .on('pagechange', ImpMobile.pageShow)
-            .on('HordeMobile:runTasks', ImpMobile.runTasks);
-
         $('#imp-mailbox-list')
             .swipebutton()
                 .on('swipebutton', 'li.imp-message', ImpMobile.swipeButtons)
             .listviewtaphold({ popupElt: $('#imp-mailbox-taphold') })
                 .on('listviewtaphold', 'li.imp-message', ImpMobile.mailboxTaphold);
 
+        ImpMobile.resetRefresh();
+
         $('#message').on('swipeleft', function() {
             $.mobile.changePage('#message-next');
         }).on('swiperight', function() {
@@ -1476,14 +1583,13 @@ var ImpMobile = {
                 break;
             }
         });
+        $('#imp-message-headers').on('expand', ImpMobile.fullHeaders);
+        $('#imp-message-atc').on('expand', ImpMobile.showAttachments);
 
         $('#compose').on('popupbeforeposition', function() {
             ImpMobile.composeMorePopup();
         });
 
-        $('#imp-message-headers').on('expand', ImpMobile.fullHeaders);
-        $('#imp-message-atc').on('expand', ImpMobile.showAttachments);
-
         if (!IMP.conf.disable_compose) {
             $('#imp-compose-upload-container a').on('click', function(e) {
                 $('#imp-compose-upload-container input').trigger('click');
@@ -1499,22 +1605,19 @@ var ImpMobile = {
             });
 
             $.each([ 'to', 'cc' ], function(undefined, v) {
-                $('#imp-compose-' + v).autocomplete({
-                    callback: function(e) {
-                        ImpMobile.addAddress(v, $.trim($(e.currentTarget).text()));
-                        $('#imp-compose-' + v).val('');
-                    },
-                    link: '#',
-                    minLength: 3,
-                    source: 'smartmobileAutocomplete',
-                    target: $('#imp-compose-' + v + '-suggestions')
-                });
-            });
-        }
-
-        if (IMP.conf.allow_folders) {
-            $('#imp-copymove-list').change(function(e) {
-                $.fn[$(this[this.selectedIndex]).hasClass('flistCreate') ? 'show' : 'hide'].call($('#imp-copymove-newdiv'));
+                $('#imp-compose-' + v)
+                    .autocomplete({
+                        callback: function(e) {
+                            ImpMobile.addAddress(v, $.trim($(e.currentTarget).text()));
+                            $('#imp-compose-' + v).val('');
+                        },
+                        link: '#',
+                        minLength: 3,
+                        source: 'smartmobileAutocomplete',
+                        target: $('#imp-compose-' + v + '-suggestions')
+                    })
+                    // textchange plugin requires bind()
+                    .bind('textchange', ImpMobile.processComposeAddress);
             });
         }
     }
@@ -1523,12 +1626,10 @@ var ImpMobile = {
 
 // JQuery Mobile setup
 $(ImpMobile.onDocumentReady);
-
-$(document).on('mobileinit', function() {
-    $.mobile.buttonMarkup.hoverDelay = 80;
-    $.mobile.defaultPageTransition = 'none';
-    $.event.special.tap.tapholdThreshold = 600;
-});
+$(document).on('pagebeforechange', ImpMobile.toPage)
+    .on('pagebeforeshow', ImpMobile.beforeShow)
+    .on('pagechange', ImpMobile.pageShow)
+    .on('HordeMobile:runTasks', ImpMobile.runTasks);
 
 var ImpMobileMbox = {
     // Vars used: cacheid, label, readonly
@@ -1552,7 +1653,7 @@ var ImpMobileMbox = {
     {
         var ids = [];
 
-        $.each(this.data, function(key, value) {
+        $.each(this.data, function(key) {
             ids.push(key);
         });
 
@@ -1580,7 +1681,7 @@ var ImpMobileMbox = {
             return { sort: value, buid: key };
         }).sort(function(a, b) {
             return (a.sort < b.sort) ? -1 : 1;
-        }), function(value, key) {
+        }), function(value) {
             return {
                 buid: value.buid,
                 data: mbox_data[value.buid]
@@ -1617,7 +1718,9 @@ var IMP_JS = {
         id = $('#' + id);
         var d = id.get(0).contentWindow.document;
 
-        id.one('load', function() { IMP_JS.iframeResize(id); });
+        id.one('load', function() {
+            window.setTimeout(function() { IMP_JS.iframeResize(id); }, 0);
+        });
 
         d.open();
         d.write(data);
@@ -1652,6 +1755,7 @@ var IMP_JS = {
         $.each(doc.find('[htmlimgblocked]'), function(k, v) {
             v = $(v);
             var src = v.attr('htmlimgblocked');
+            v.removeAttr('htmlimgblocked');
 
             if (v.attr('src')) {
                 v.one('load', function() { IMP_JS.iframeResize(iframe); });
@@ -1667,9 +1771,16 @@ var IMP_JS = {
             }
         });
 
+        $.each(doc.find('[htmlimgblocked_srcset]'), function(k, v) {
+            v = $(v);
+            v.attr('srcset', v.attr('htmlimgblocked_srcset'));
+            v.removeAttr('htmlimgblocked_srcset');
+        });
+
         $.each(doc.find('[htmlcssblocked]'), function(k, v) {
             v = $(v);
             v.attr('href', v.attr('htmlcssblocked'));
+            v.removeAttr('htmlcssblocked');
         });
 
         doc.find('STYLE[type="text/x-imp-cssblocked"]')
