@@ -1,19 +1,19 @@
 <?php
 /**
- * Copyright 2010-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2010-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2010-2013 Horde LLC
+ * @copyright 2010-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
 
 /* Determine the base directories. */
 if (!defined('IMP_BASE')) {
-    define('IMP_BASE', __DIR__ . '/..');
+    define('IMP_BASE', realpath(__DIR__ . '/..'));
 }
 
 if (!defined('HORDE_BASE')) {
@@ -22,7 +22,7 @@ if (!defined('HORDE_BASE')) {
     if (file_exists(IMP_BASE . '/config/horde.local.php')) {
         include IMP_BASE . '/config/horde.local.php';
     } else {
-        define('HORDE_BASE', IMP_BASE . '/..');
+        define('HORDE_BASE', realpath(IMP_BASE . '/..'));
     }
 }
 
@@ -38,7 +38,7 @@ require_once HORDE_BASE . '/lib/core.php';
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2010-2013 Horde LLC
+ * @copyright 2010-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -66,7 +66,7 @@ class IMP_Application extends Horde_Registry_Application
 
     /**
      */
-    public $version = 'H5 (6.1.6)';
+    public $version = 'H5 (6.2.1)';
 
     /**
      * Server key used in logged out session.
@@ -101,9 +101,10 @@ class IMP_Application extends Horde_Registry_Application
             'IMP_Crypt_Smime' => 'IMP_Factory_Smime',
             'IMP_Flags' => 'IMP_Factory_Flags',
             'IMP_Identity' => 'IMP_Factory_Identity',
-            'IMP_Imap' => 'IMP_Factory_Imap',
-            'IMP_Imap_Tree' => 'IMP_Factory_Imaptree',
+            'IMP_Ftree' => 'IMP_Factory_Ftree',
             'IMP_Mail' => 'IMP_Factory_Mail',
+            'IMP_Mail_Autoconfig' => 'IMP_Factory_MailAutoconfig',
+            'IMP_Mailbox_SessionCache' => 'IMP_Factory_MailboxCache',
             'IMP_Maillog' => 'IMP_Factory_Maillog',
             'IMP_Prefs_Sort' => 'IMP_Factory_PrefsSort',
             'IMP_Quota' => 'IMP_Factory_Quota',
@@ -117,7 +118,7 @@ class IMP_Application extends Horde_Registry_Application
 
         /* Methods only available if admin config is set for this
          * server/login. */
-        if (empty($injector->getInstance('IMP_Imap')->config->admin)) {
+        if (empty($injector->getInstance('IMP_Factory_Imap')->create()->config->admin)) {
             $this->auth = array_diff($this->auth, array('add', 'list', 'remove'));
         }
 
@@ -137,24 +138,15 @@ class IMP_Application extends Horde_Registry_Application
      */
     protected function _init()
     {
-        global $prefs, $registry;
+        global $registry;
 
-        // Set default message character set.
-        if ($registry->getAuth()) {
-            if ($def_charset = $prefs->getValue('default_msg_charset')) {
-                Horde_Mime_Part::$defaultCharset = $def_charset;
-                Horde_Mime_Headers::$defaultCharset = $def_charset;
-            }
-
-            // Always use Windows-1252 in place of ISO-8859-1 for MIME
-            // decoding.
-            Horde_Mime::$decodeWindows1252 = true;
-        }
+        // Always use Windows-1252 in place of ISO-8859-1 for MIME decoding.
+        Horde_Mime::$decodeWindows1252 = true;
 
         if (($registry->initialApp == 'imp') &&
             !empty($this->initParams['impmode']) &&
             ($this->initParams['impmode'] != $registry->getView())) {
-            IMP_Auth::getInitialPage()->url->redirect();
+            IMP::getInitialPage()->url->redirect();
         }
     }
 
@@ -162,20 +154,27 @@ class IMP_Application extends Horde_Registry_Application
      */
     public function logout()
     {
+        global $injector;
+
         /* Clean up dangling IMP_Compose objects. */
-        foreach (array_keys($GLOBALS['session']->get('imp', 'compose_cache', Horde_Session::TYPE_ARRAY)) as $key) {
-            $GLOBALS['injector']->getInstance('IMP_Factory_Compose')->create($key)->destroy('cancel');
+        foreach ($injector->getInstance('IMP_Factory_Compose')->getAllObs() as $val) {
+            $val->destroy('cancel');
         }
 
-        /* No need to keep Tree object in cache - it will be recreated next
-         * login. */
-        if ($treeob = $GLOBALS['session']->get('imp', 'treeob')) {
-            $GLOBALS['injector']->getInstance('Horde_Cache')->expire($treeob);
-        }
+        /* Destroy any IMP_Mailbox_List cached entries, since they will not
+         * be used in the next session. */
+        $injector->getInstance('IMP_Factory_MailboxList')->expireAll();
 
         /* Grab the current server from the session to correctly populate
          * login form. */
-        $this->_oldserver = $GLOBALS['injector']->getInstance('IMP_Imap')->server_key;
+        $this->_oldserver = $injector->getInstance('IMP_Factory_Imap')->create()->server_key;
+    }
+
+    /**
+     */
+    public function getInitialPage()
+    {
+        return strval(IMP::getInitialPage()->url->setRaw(true));
     }
 
     /* Horde permissions. */
@@ -200,9 +199,9 @@ class IMP_Application extends Horde_Registry_Application
      */
     public function menu($menu)
     {
-        global $injector, $prefs, $registry;
+        global $injector, $prefs, $registry, $session;
 
-        $imp_imap = $injector->getInstance('IMP_Imap');
+        $imp_imap = $injector->getInstance('IMP_Factory_Imap')->create();
 
         if ($imp_imap->access(IMP_Imap::ACCESS_TRASH) &&
             $prefs->getValue('use_trash') &&
@@ -212,9 +211,9 @@ class IMP_Application extends Horde_Registry_Application
             $menu->addArray(array(
                 'class' => '__noselection',
                 'icon' => 'imp-empty-trash',
-                'onclick' => 'return window.confirm(' . Horde_Serialize::serialize(_("Are you sure you wish to empty your trash mailbox?"), Horde_Serialize::JSON, 'UTF-8') . ')',
+                'onclick' => 'return window.confirm(' . json_encode(_("Are you sure you wish to empty your trash mailbox?")) . ')',
                 'text' => _("Empty _Trash"),
-                'url' => $trash->url('mailbox')->add(array('actionID' => 'empty_mailbox', 'mailbox_token' => $injector->getInstance('Horde_Token')->get('imp.mailbox')))
+                'url' => $trash->url('mailbox')->add(array('actionID' => 'empty_mailbox', 'token' => $session->getToken()))
             ));
         }
 
@@ -225,9 +224,9 @@ class IMP_Application extends Horde_Registry_Application
             $menu->addArray(array(
                 'class' => '__noselection',
                 'icon' =>  'imp-empty-spam',
-                'onclick' => 'return window.confirm(' . Horde_Serialize::serialize(_("Are you sure you wish to empty your spam mailbox?"), Horde_Serialize::JSON, 'UTF-8') . ')',
+                'onclick' => 'return window.confirm(' . json_encode(_("Are you sure you wish to empty your spam mailbox?")) . ')',
                 'text' => _("Empty _Spam"),
-                'url' => $spam->url('mailbox')->add(array('actionID' => 'empty_mailbox', 'mailbox_token' => $injector->getInstance('Horde_Token')->get('imp.mailbox')))
+                'url' => $spam->url('mailbox')->add(array('actionID' => 'empty_mailbox', 'token' => $session->getToken()))
             ));
         }
 
@@ -263,20 +262,28 @@ class IMP_Application extends Horde_Registry_Application
      */
     public function sidebar($sidebar)
     {
+        global $injector;
+
         if (IMP_Compose::canCompose()) {
             $clink = new IMP_Compose_Link();
             $sidebar->addNewButton(_("_New Message"), $clink->link());
         }
 
         /* Folders. */
-        if ($GLOBALS['injector']->getInstance('IMP_Imap')->access(IMP_Imap::ACCESS_FOLDERS)) {
+        if ($injector->getInstance('IMP_Factory_Imap')->create()->access(IMP_Imap::ACCESS_FOLDERS)) {
             try {
-                $tree = $GLOBALS['injector']
-                    ->getInstance('Horde_Core_Factory_Tree')
+                $tree = $injector->getInstance('Horde_Core_Factory_Tree')
                     ->create('imp_menu', 'Horde_Tree_Renderer_Sidebar', array('nosession' => true));
-                $imaptree = $GLOBALS['injector']->getInstance('IMP_Imap_Tree');
-                $imaptree->setIteratorFilter(IMP_Imap_Tree::FLIST_VFOLDER);
-                $tree = $imaptree->createTree($tree, array(
+
+                $ftree = $injector->getInstance('IMP_Ftree');
+                $iterator = new IMP_Ftree_IteratorFilter($ftree);
+                $iterator->add(array(
+                    $iterator::REMOTE,
+                    $iterator::VFOLDER
+                ));
+
+                $tree = $ftree->createTree($tree, array(
+                    'iterator' => $iterator,
                     'open' => false,
                     'poll_info' => true
                 ));
@@ -429,7 +436,7 @@ class IMP_Application extends Horde_Registry_Application
             ));
         }
 
-        $imp_imap = $injector->getInstance('IMP_Imap');
+        $imp_imap = $injector->getInstance('IMP_Factory_Imap')->create();
         if ($imp_imap->access(IMP_Imap::ACCESS_SEARCH)) {
             $onclick = null;
             switch ($registry->getView()) {
@@ -465,7 +472,7 @@ class IMP_Application extends Horde_Registry_Application
      */
     public function changeLanguage()
     {
-        $GLOBALS['injector']->getInstance('IMP_Imap_Tree')->init();
+        $GLOBALS['injector']->getInstance('IMP_Ftree')->init();
     }
 
     /* NoSQL methods. */
@@ -478,14 +485,30 @@ class IMP_Application extends Horde_Registry_Application
 
         $out = array();
 
-        $ob = $injector->getInstance('IMP_Imap')->config->cache_params;
-        if ($ob['backend'] instanceof Horde_Imap_Client_Cache_Backend_Mongo) {
-            $out[] = $ob['backend'];
-        }
+        $backends = array(
+            'Horde_Imap_Client_Cache_Backend_Mongo' => function() use ($injector) {
+                $backend = $injector
+                    ->getInstance('IMP_Factory_Imap')
+                    ->create()
+                    ->config
+                    ->cache_params['backend'];
+                if (isset($backend->backend)) {
+                    return $backend->backend;
+                }
+            },
+            'IMP_Sentmail_Mongo' => function() use ($injector) {
+                return $injector->getInstance('IMP_Sentmail');
+            },
+        );
 
-        $ob = $injector->getInstance('IMP_Sentmail');
-        if ($ob instanceof IMP_Sentmail_Mongo) {
-            $out[] = $ob;
+        foreach ($backends as $key => $func) {
+            try {
+                $val = $func();
+                if ($val instanceof $key) {
+                    $drivers[] = $val;
+                }
+            } catch (Horde_Exception $e) {
+            }
         }
 
         return $out;
@@ -580,7 +603,10 @@ class IMP_Application extends Horde_Registry_Application
     public function exceptionHandler(Exception $e)
     {
         if ($e instanceof Horde_Imap_Client_Exception) {
-            $e = new Horde_Exception_AuthenticationFailure($e->getMessage(), Horde_Auth::REASON_MESSAGE);
+            $e = new Horde_Exception_AuthenticationFailure(
+                $e->getMessage(),
+                Horde_Auth::REASON_MESSAGE
+            );
         }
 
         Horde_ErrorHandler::fatal($e);
