@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 1999-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 1999-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 1999-2013 Horde LLC
+ * @copyright 1999-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -17,7 +17,7 @@
  * @author    Chuck Hagenbuch <chuck@horde.org>
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 1999-2013 Horde LLC
+ * @copyright 1999-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -29,13 +29,17 @@ class IMP_Basic_Mailbox extends IMP_Basic_Base
      */
     protected function _init()
     {
-        global $injector, $notification, $page_output, $prefs, $registry;
+        global $injector, $notification, $page_output, $prefs, $registry, $session;
 
         $mailbox = $this->indices->mailbox;
 
         /* Call the mailbox redirection hook, if requested. */
         try {
-            $redirect = Horde::callHook('mbox_redirect', array($mailbox), 'imp');
+            $redirect = $injector->getInstance('Horde_Core_Hooks')->callHook(
+                'mbox_redirect',
+                'imp',
+                array($mailbox)
+            );
             if (!empty($redirect)) {
                 Horde::url($redirect, true)->redirect();
             }
@@ -44,17 +48,16 @@ class IMP_Basic_Mailbox extends IMP_Basic_Base
         $mailbox_url = Horde::url('basic.php')->add('page', 'mailbox');
         $mailbox_imp_url = $mailbox->url('mailbox')->add('newmail', 1);
 
-        $horde_token = $injector->getInstance('Horde_Token');
         $imp_flags = $injector->getInstance('IMP_Flags');
-        $imp_imap = $injector->getInstance('IMP_Imap');
+        $imp_imap = $mailbox->imp_imap;
         $imp_search = $injector->getInstance('IMP_Search');
 
         /* Run through the action handlers */
         if (($actionID = $this->vars->actionID) &&
             ($actionID != 'message_missing')) {
             try {
-                $horde_token->validate($this->vars->mailbox_token, 'imp.mailbox');
-            } catch (Horde_Token_Exception $e) {
+                $session->checkToken($this->vars->token);
+            } catch (Horde_Exception $e) {
                 $notification->push($e);
                 $actionID = null;
             }
@@ -222,7 +225,7 @@ class IMP_Basic_Mailbox extends IMP_Basic_Base
         }
 
         /* Token to use in requests. */
-        $mailbox_token = $horde_token->get('imp.mailbox');
+        $token = $session->getToken();
         $search_mbox = $mailbox->search;
 
         /* Deal with filter options. */
@@ -233,16 +236,22 @@ class IMP_Basic_Mailbox extends IMP_Basic_Base
             ($prefs->getValue('filter_any_mailbox') && !$search_mbox))) {
             $filter_url = $mailbox_imp_url->copy()->add(array(
                 'actionID' => 'filter',
-                'mailbox_token' => $mailbox_token
+                'token' => $token
             ));
         }
 
         /* Generate folder options list. */
         if ($imp_imap->access(IMP_Imap::ACCESS_FOLDERS)) {
-            $folder_options = IMP::flistSelect(array(
+            $iterator = new IMP_Ftree_IteratorFilter(
+                $injector->getInstance('IMP_Ftree')
+            );
+            $iterator->add($iterator::NONIMAP);
+
+            $folder_options = new IMP_Ftree_Select(array(
                 'heading' => _("Messages to"),
                 'inc_notepads' => true,
                 'inc_tasklists' => true,
+                'iterator' => $iterator,
                 'new_mbox' => true
             ));
         }
@@ -312,17 +321,15 @@ class IMP_Basic_Mailbox extends IMP_Basic_Base
         if ($mailbox->inbox) {
             $rss_box = '';
         } else {
-            $rss_box = $mailbox;
-            $ns_info = $imp_imap->getNamespace($mailbox);
-            if ($ns_info !== null) {
-                if (!empty($ns_info['name']) &&
-                    ($ns_info['type'] == Horde_Imap_Client::NS_PERSONAL) &&
-                    substr($mailbox, 0, strlen($ns_info['name'])) == $ns_info['name']) {
-                    $rss_box = substr($mailbox, strlen($ns_info['name']));
-                }
-                $rss_box = str_replace(rawurlencode($ns_info['delimiter']), '/', rawurlencode($ns_info['delimiter'] . $rss_box));
-            } else {
+            $ns_info = $mailbox->namespace_info;
+            if (is_null($ns_info)) {
                 $rss_box = null;
+            } else {
+                $rss_box = str_replace(
+                    rawurlencode($ns_info->delimiter),
+                    '/',
+                    rawurlencode($ns_info->delimiter . (($ns_info->type == $ns_info::NS_PERSONAL) ? $ns_info->stripNamespace($mailbox) : $mailbox))
+                );
             }
         }
 
@@ -362,8 +369,7 @@ class IMP_Basic_Mailbox extends IMP_Basic_Base
                 'selectonlyone' => _("You must select only one message for this action."),
                 'spam_report' => _("Are you sure you wish to report this message as spam?"),
                 'submit' => _("You must select at least one message first."),
-                'target_mbox' => _("You must select a target mailbox first."),
-                'yes' => _("Yes")
+                'target_mbox' => _("You must select a target mailbox first.")
             ),
             'ImpMailbox.unread' => intval($unread)
         ));
@@ -396,7 +402,7 @@ class IMP_Basic_Mailbox extends IMP_Basic_Base
 
         $page_output->addScriptFile('hordecore.js', 'horde');
         $page_output->addScriptFile('mailbox.js');
-        $page_output->addScriptPackage('Dialog');
+        $page_output->addScriptPackage('Horde_Core_Script_Package_Dialog');
 
         $page_output->metaRefresh($prefs->getValue('refresh_time'), $refresh_url);
 
@@ -441,7 +447,7 @@ class IMP_Basic_Mailbox extends IMP_Basic_Base
         if ($mailbox->access_empty) {
             $hdr_view->empty = $mailbox_imp_url->copy()->add(array(
                 'actionID' => 'empty_mailbox',
-                'mailbox_token' => $mailbox_token
+                'token' => $token
             ));
         }
 
@@ -456,7 +462,7 @@ class IMP_Basic_Mailbox extends IMP_Basic_Base
                 $del_view->hide = Horde::widget(array(
                     'url' => $refresh_url->copy()->add(array(
                         'actionID' => 'hide_deleted',
-                        'mailbox_token' => $mailbox_token
+                        'token' => $token
                     )),
                     'class' => 'hideAction',
                     'title' => $deleted_prompt
@@ -465,7 +471,7 @@ class IMP_Basic_Mailbox extends IMP_Basic_Base
                     $del_view->purge = Horde::widget(array(
                         'url' => $refresh_url->copy()->add(array(
                             'actionID' => 'expunge_mailbox',
-                            'mailbox_token' => $mailbox_token
+                            'token' => $token
                         )),
                         'class' => 'purgeAction',
                         'title' => _("Pur_ge Deleted")
@@ -526,8 +532,11 @@ class IMP_Basic_Mailbox extends IMP_Basic_Base
 
             if (!$search_mbox && $mailbox->access_filters) {
                 $filters = array();
-                $imp_search->setIteratorFilter(IMP_Search::LIST_FILTER);
-                foreach ($imp_search as $val) {
+                $iterator = IMP_Search_IteratorFilter::create(
+                    IMP_Search_IteratorFilter::FILTER
+                );
+
+                foreach ($iterator as $val) {
                     $filters[] = array(
                         'l' => $val->label,
                         'v' => IMP_Mailbox::formTo($val)
@@ -604,7 +613,7 @@ class IMP_Basic_Mailbox extends IMP_Basic_Base
                     $mboxactions[] = Horde::widget(array(
                         'url' => $mailbox_link->copy()->add(array(
                             'actionID' => 'hide_deleted',
-                            'mailbox_token' => $mailbox_token
+                            'token' => $token
                         )),
                         'class' => 'hideAction',
                         'title' => $deleted_prompt
@@ -613,7 +622,7 @@ class IMP_Basic_Mailbox extends IMP_Basic_Base
                 $mboxactions[] = Horde::widget(array(
                     'url' => $mailbox_link->copy()->add(array(
                         'actionID' => 'expunge_mailbox',
-                        'mailbox_token' => $mailbox_token
+                        'token' => $token
                     )),
                     'class' => 'purgeAction',
                     'title' => _("Pur_ge Deleted")
@@ -626,7 +635,7 @@ class IMP_Basic_Mailbox extends IMP_Basic_Base
                     'url' => $mailbox_imp_url->copy()->add(array(
                         'sortby' => Horde_Imap_Client::SORT_SEQUENCE,
                         'actionID' => 'change_sort',
-                        'mailbox_token' => $mailbox_token
+                        'token' => $token
                     )),
                     'title' => _("Clear Sort")
                 ));
@@ -767,7 +776,7 @@ class IMP_Basic_Mailbox extends IMP_Basic_Base
             $headers[$standard]['altsort'] = Horde::widget(array(
                 'url' => $mailbox_imp_url->copy()->add(array(
                     'actionID' => 'change_sort',
-                    'mailbox_token' => $mailbox_token,
+                    'token' => $token,
                     'sortby' => $extra
                 )),
                 'title' => $headers[$extra]['text']
@@ -787,7 +796,7 @@ class IMP_Basic_Mailbox extends IMP_Basic_Base
                         'sortby' => $key,
                         'sortdir' => intval(!$sortpref->sortdir),
                         'actionID' => 'change_sort',
-                        'mailbox_token' => $mailbox_token
+                        'token' => $token
                     ));
                     $ptr['change_sort_link'] = Horde::link($tmp, $val['stext'], null, null, null, $val['stext']) . $csl_icon . '</a>';
                     $ptr['change_sort_widget'] = Horde::widget(array('url' => $tmp, 'title' => $val['text']));
@@ -799,7 +808,7 @@ class IMP_Basic_Mailbox extends IMP_Basic_Base
                     : Horde::widget(array(
                         'url' => $mailbox_imp_url->copy()->add(array(
                             'actionID' => 'change_sort',
-                            'mailbox_token' => $mailbox_token,
+                            'token' => $token,
                             'sortby' => $key
                         )),
                         'title' => $val['text']
@@ -811,9 +820,9 @@ class IMP_Basic_Mailbox extends IMP_Basic_Base
         /* Output the form start. */
         $f_view = clone $view;
         $f_view->mailbox = $mailbox->form_to;
-        $f_view->mailbox_token = $mailbox_token;
         $f_view->mailbox_url = $mailbox_url;
         $f_view->page = $pageOb['page'];
+        $f_view->token = $token;
         $this->output .= $f_view->render('form_start');
 
         /* Prepare the message headers template. */
@@ -937,7 +946,7 @@ class IMP_Basic_Mailbox extends IMP_Basic_Base
                 }
                 break;
 
-            case 1:
+            default:
                 $from_uri = $mailbox->url('message', $msg['buid']);
                 $msg['from'] = Horde::link($from_uri) . $msg['from'] . '</a>';
                 break;
@@ -995,7 +1004,11 @@ class IMP_Basic_Mailbox extends IMP_Basic_Base
     {
         /* Allow user to alter template array. */
         try {
-            $msgs = Horde::callHook('mailboxarray', array($msgs), 'imp');
+            $msgs = $GLOBALS['injector']->getInstance('Horde_Core_Hooks')->callHook(
+                'mailboxarray',
+                'imp',
+                array($msgs)
+            );
         } catch (Horde_Exception_HookNotSet $e) {}
 
         $view->messages = $msgs;
