@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2004-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2004-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2004-2013 Horde LLC
+ * @copyright 2004-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -17,7 +17,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2004-2013 Horde LLC
+ * @copyright 2004-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -27,9 +27,8 @@ class IMP_Basic_Thread extends IMP_Basic_Base
      */
     protected function _init()
     {
-        global $injector, $notification, $page_output, $registry;
+        global $injector, $notification, $page_output, $registry, $session;
 
-        $imp_imap = $injector->getInstance('IMP_Imap');
         $imp_mailbox = $this->indices->mailbox->list_ob;
 
         switch ($mode = $this->vars->get('mode', 'thread')) {
@@ -74,7 +73,10 @@ class IMP_Basic_Thread extends IMP_Basic_Base
         /* Force images to show in HTML data. */
         $injector->getInstance('IMP_Images')->alwaysShow = true;
 
+        $multiple = (count($imp_indices) > 1);
+
         foreach ($imp_indices as $ob) {
+            $imp_imap = $ob->mbox->imp_imap;
             $fetch_res = $imp_imap->fetch($ob->mbox, $query, array(
                 'ids' => $imp_imap->getIdsOb($ob->uids)
             ));
@@ -114,14 +116,23 @@ class IMP_Basic_Thread extends IMP_Basic_Base
                 }
 
                 $subject_header = htmlspecialchars($envelope->subject, ENT_COMPAT, 'UTF-8');
-                if (($mode == 'thread') && empty($subject)) {
-                    $subject = preg_replace('/^re:\s*/i', '', $subject_header);
-                }
 
-                /* Create links to current message and mailbox. */
-                $curr_msg['link'] = ($mode == 'thread')
-                    ? Horde::widget(array('url' => '#display', 'title' => _("Thread List"), 'nocheck' => true))
-                    : Horde::widget(array('url' => '#display', 'title' => _("Back to Multiple Message View Index"), 'nocheck' => true));
+                switch ($mode) {
+                case 'thread':
+                    if (empty($subject)) {
+                        $subject = preg_replace('/^re:\s*/i', '', $subject_header);
+                    }
+                    $curr_msg['link'] = $multiple
+                        ? Horde::widget(array('url' => '#display', 'title' => _("Thread List"), 'nocheck' => true))
+                        : '';
+                    $curr_tree['subject'] = $imp_mailbox->getThreadOb($imp_mailbox->getArrayIndex($fetch_res[$idx]->getUid(), $ob->mbox) + 1)->img;
+                    break;
+
+                default:
+                    $curr_msg['link'] = Horde::widget(array('url' => '#display', 'title' => _("Back to Multiple Message View Index"), 'nocheck' => true));
+                    $curr_tree['subject'] = '';
+                    break;
+                }
 
                 switch ($registry->getView()) {
                 case $registry::VIEW_BASIC:
@@ -130,9 +141,6 @@ class IMP_Basic_Thread extends IMP_Basic_Base
                     break;
                 }
 
-                $curr_tree['subject'] = ($mode == 'thread')
-                    ? $imp_mailbox->getThreadOb($imp_mailbox->getArrayIndex($fetch_res[$idx]->getUid(), $ob->mbox) + 1)->img
-                    : ' ';
                 $curr_tree['subject'] .= Horde::link('#i' . $idx) . Horde_String::truncate($subject_header, 60) . '</a> (' . $addr . ')';
 
                 $msgs[] = $curr_msg;
@@ -159,12 +167,12 @@ class IMP_Basic_Thread extends IMP_Basic_Base
                 $delete_link = $this->indices->mailbox->url('mailbox')->add(array(
                     'actionID' => 'delete_messages',
                     'indices' => strval($imp_indices),
-                    'mailbox_token' => $injector->getInstance('Horde_Token')->get('imp.mailbox'),
+                    'token' => $session->getToken(),
                     'start' => $imp_mailbox->getArrayIndex(end($uid_list))
                 ));
                 $view->delete = Horde::link($delete_link, _("Delete Thread"), null, null, null, null, null, array('id' => 'threaddelete'));
                 $page_output->addInlineScript(array(
-                    '$("threaddelete").observe("click", function(e) { if (!window.confirm(' . Horde_Serialize::serialize(_("Are you sure you want to delete all messages in this thread?"), Horde_Serialize::JSON) . ')) { e.stop(); } })'
+                    '$("threaddelete").observe("click", function(e) { if (!window.confirm(' . json_encode(_("Are you sure you want to delete all messages in this thread?")) . ')) { e.stop(); } })'
                 ), true);
                 break;
             }
