@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2008-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2008-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2008-2013 Horde LLC
+ * @copyright 2008-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -17,13 +17,16 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2008-2013 Horde LLC
+ * @copyright 2008-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  *
  * @property-read boolean $changed  If true, this object has changed.
- * @property-read IMP_Imap_Config $config  Backend config settings.
- * @property-read boolean $init  Has the base IMAP object been initialized?
+ * @property-read Horde_Imap_Client_Base $client_ob  The IMAP client object.
+ * @property-read IMP_Imap_Config $config  Base backend config settings.
+ * @property-read boolean $init  Has the IMAP object been initialized?
+ * @property-read integer $max_compose_bodysize  The maximum size (in bytes)
+ *                                               of the compose message body.
  * @property-read integer $max_compose_recipients  The maximum number of
  *                                                 recipients to send to per
  *                                                 compose message.
@@ -34,6 +37,7 @@
  *                                            a user can create.
  * @property-read string $server_key  Server key used to login.
  * @property-read string $thread_algo  The threading algorithm to use.
+ * @property-read Horde_Imap_Client_Url $url  A URL object.
  */
 class IMP_Imap implements Serializable
 {
@@ -45,10 +49,16 @@ class IMP_Imap implements Serializable
     const ACCESS_TRASH = 5;
     const ACCESS_CREATEMBOX = 6;
     const ACCESS_CREATEMBOX_MAX = 7;
+    const ACCESS_COMPOSE_BODYSIZE = 13;
     const ACCESS_COMPOSE_RECIPIENTS = 8;
     const ACCESS_COMPOSE_TIMELIMIT = 9;
     const ACCESS_ACL = 10;
     const ACCESS_DRAFTS = 11;
+    const ACCESS_REMOTE = 12;
+    const ACCESS_IMPORT = 14;
+
+    /* Default namespace. */
+    const NS_DEFAULT = "\0default";
 
     /**
      * Cached backend configuration.
@@ -72,11 +82,18 @@ class IMP_Imap implements Serializable
     protected $_config;
 
     /**
-     * The Horde_Imap_Client object.
+     * Object identifier.
      *
-     * @var Horde_Imap_Client
+     * @var string
      */
-    protected $_ob = null;
+    protected $_id;
+
+    /**
+     * The IMAP client object.
+     *
+     * @var Horde_Imap_Client_Base
+     */
+    protected $_ob;
 
     /**
      * Temporary data cache (destroyed at end of request).
@@ -85,13 +102,28 @@ class IMP_Imap implements Serializable
      */
     protected $_temp = array();
 
+    /**
+     * Constructor.
+     *
+     * @param string $id  Object identifier.
+     */
+    public function __construct($id)
+    {
+        $this->_id = strval($id);
+    }
+
     /**
      */
     public function __get($key)
     {
         switch ($key) {
         case 'changed':
-            return $this->_changed || ($this->_ob && $this->_ob->changed);
+            return $this->_changed;
+
+        case 'client_ob':
+            return $this->init
+                ? $this->_ob
+                : null;
 
         case 'config':
             return isset($this->_config)
@@ -99,8 +131,9 @@ class IMP_Imap implements Serializable
                 : new Horde_Support_Stub();
 
         case 'init':
-            return !is_null($this->_ob);
+            return isset($this->_ob);
 
+        case 'max_compose_bodysize':
         case 'max_compose_recipients':
         case 'max_compose_timelimit':
             $perm = $GLOBALS['injector']->getInstance('Horde_Perms')->getPermissions('imp:' . str_replace('max_compose', 'max', $key), $GLOBALS['registry']->getAuth());
@@ -134,9 +167,25 @@ class IMP_Imap implements Serializable
             $this->_changed = true;
 
             return $thread;
+
+        case 'url':
+            $url = new Horde_Imap_Client_Url();
+            if ($this->init) {
+                $url->hostspec = $this->getParam('hostspec');
+                $url->port = $this->getParam('port');
+                $url->protocol = $this->isImap() ? 'imap' : 'pop';
+            }
+            return $url;
         }
     }
 
+    /**
+     */
+    public function __toString()
+    {
+        return $this->_id;
+    }
+
     /**
      * Get the full permission name for a permission.
      *
@@ -144,108 +193,125 @@ class IMP_Imap implements Serializable
      *
      * @return string  The full (backend-specific) permission name.
      */
-    private function _getPerm($perm)
+    protected function _getPerm($perm)
     {
         return 'backends:' . ($this->init ? $this->server_key . ':' : '') . $perm;
     }
 
     /**
-     * Returns the IMAP Client object.
+     * Determine if this is a connection to an IMAP server.
      *
-     * @param IMP_Mailbox $mbox  Get the IMAP client for a given mailbox. If
-     *                           null, returns the IMAP client for the login
-     *                           backend.
-     *
-     * @return Horde_Imap_Client_Base  An IMAP Client object.
+     * @return boolean  True if connected to IMAP server.
      */
-    public function getOb($mbox = null)
+    public function isImap()
     {
-        return $this->_ob;
+        return ($this->init &&
+                ($this->_ob instanceof Horde_Imap_Client_Socket));
     }
 
     /**
      * Determine if this is a connection to an IMAP server.
      *
-     * @param IMP_Mailbox $mbox  Specifically check this mailbox. Otherwise,
-     *                           checks the base IMAP objecct.
-     *
-     * @return boolean  True if connected to IMAP server, false if connected
-     *                  to a POP3 server.
+     * @return boolean  True if connected to IMAP server.
      */
-    public function isImap($mbox = null)
+    public function isPop3()
     {
-        return !$this->_ob || ($this->_ob instanceof Horde_Imap_Client_Socket);
+        return ($this->init &&
+                ($this->_ob instanceof Horde_Imap_Client_Socket_Pop3));
     }
 
     /**
-     * Is sorting available for a mailbox?
-     *
-     * @param IMP_Mailbox $mbox  The mailbox to query.
+     * Is mailbox sorting available?
      *
      * @return boolean  True if sorting is available.
      */
-    public function canSort(IMP_Mailbox $mbox)
+    public function canSort()
     {
         return ($this->config->sort_force ||
-                $this->getOb($mbox)->queryCapability('SORT'));
+                $this->_ob->queryCapability('SORT'));
     }
 
     /**
-     * Create a new Horde_Imap_Client object.
+     * Create the base Horde_Imap_Client object (from an entry in
+     * backends.php).
      *
      * @param string $username  The username to authenticate with.
      * @param string $password  The password to authenticate with.
-     * @param string $key       Create a new object using this server key.
+     * @param string $skey      Create a new object using this server key.
      *
      * @return Horde_Imap_Client_Base  Client object.
      * @throws IMP_Imap_Exception
      */
-    public function createImapObject($username, $password, $key)
+    public function createBaseImapObject($username, $password, $skey)
     {
-        if (!is_null($this->_ob)) {
-            return $this->_ob;
+        if ($this->init) {
+            return $this->client_ob;
         }
 
-        if (($config = $this->loadServerConfig($key)) === false) {
+        if (($config = $this->loadServerConfig($skey)) === false) {
             $error = new IMP_Imap_Exception('Could not load server configuration.');
             Horde::log($error);
             throw $error;
         }
 
         $imap_config = array(
-            'cache' => $config->cache_params,
-            'capability_ignore' => $config->capability_ignore,
-            'comparator' => $config->comparator,
-            'debug' => $config->debug,
-            'debug_literal' => $config->debug_raw,
             'hostspec' => $config->hostspec,
             'id' => $config->id,
-            'lang' => $config->lang,
             'password' => new IMP_Imap_Password($password),
             'port' => $config->port,
             'secure' => (($secure = $config->secure) ? $secure : false),
-            'timeout' => $config->timeout,
             'username' => $username,
             // IMP specific config
-            'imp:backend' => $key
-            // 'imp:login' - Set in __call()
-            // 'imp:nsdefault' - Set in defaultNamespace()
+            'imp:backend' => $skey
         );
 
+        /* Needed here to set config information in createImapObject(). */
+        $this->_config = $config;
+
         try {
-            $ob = ($config->protocol == 'imap')
-                ? new Horde_Imap_Client_Socket($imap_config)
-                : new Horde_Imap_Client_Socket_Pop3($imap_config);
-        } catch (Horde_Imap_Client_Exception $e) {
-            $error = new IMP_Imap_Exception($e);
-            Horde::log($error);
-            throw $error;
+            return $this->createImapObject($imap_config, ($config->protocol == 'imap'));
+        } catch (IMP_Imap_Exception $e) {
+            unset($this->_config);
+            throw $e;
+        }
+    }
+
+    /**
+     * Create a Horde_Imap_Client object.
+     *
+     * @param array $config  The IMAP configuration.
+     * @param boolean $imap  True if IMAP connection, false if POP3.
+     *
+     * @return Horde_Imap_Client_Base  Client object.
+     * @throws IMP_Imap_Exception
+     */
+    public function createImapObject($config, $imap = true)
+    {
+        if ($this->init) {
+            return $this->_ob;
         }
 
-        $this->_config = $config;
-        $this->_ob = $ob;
+        $sconfig = $this->config;
+        $config = array_merge(array(
+            'cache' => $sconfig->cache_params,
+            'capability_ignore' => $sconfig->capability_ignore,
+            'comparator' => $sconfig->comparator,
+            'debug' => $sconfig->debug,
+            'debug_literal' => $sconfig->debug_raw,
+            'lang' => $sconfig->lang,
+            'timeout' => $sconfig->timeout,
+            // 'imp:login' - Set in __call()
+        ), $config);
 
-        return $ob;
+        try {
+            $this->_ob = $imap
+                ? new Horde_Imap_Client_Socket($config)
+                : new Horde_Imap_Client_Socket_Pop3($config);
+            return $this->_ob;
+        } catch (Horde_Imap_Client_Exception $e) {
+            Horde::log($e->raw_msg);
+            throw new IMP_Imap_Exception($e);
+        }
     }
 
     /**
@@ -284,6 +350,7 @@ class IMP_Imap implements Serializable
             $prefs->setLocked(IMP_Mailbox::MBOX_TRASH, true);
             break;
         }
+
         $this->updateFetchIgnore();
     }
 
@@ -315,7 +382,7 @@ class IMP_Imap implements Serializable
     {
         global $injector;
 
-        if (!$this->_ob) {
+        if (!$this->init) {
             return false;
         }
 
@@ -333,6 +400,7 @@ class IMP_Imap implements Serializable
 
         case self::ACCESS_DRAFTS:
         case self::ACCESS_FLAGS:
+        case self::ACCESS_IMPORT:
         case self::ACCESS_SEARCH:
         case self::ACCESS_UNSEEN:
             return $this->isImap();
@@ -341,6 +409,9 @@ class IMP_Imap implements Serializable
         case self::ACCESS_TRASH:
             return ($this->isImap() &&
                     $injector->getInstance('Horde_Core_Perms')->hasAppPermission($this->_getPerm('allow_folders')));
+
+        case self::ACCESS_REMOTE:
+            return $injector->getInstance('Horde_Core_Perms')->hasAppPermission($this->_getPerm('allow_remote'));
         }
 
         return false;
@@ -349,103 +420,75 @@ class IMP_Imap implements Serializable
     /**
      * Checks compose access rights for a server.
      *
-     * @param integer $right        Access right.
-     * @param integer $email_count  The number of e-mail recipients.
+     * @param integer $right  Access right.
+     * @param integer $data   Data required to check the rights:
+     * <pre>
+     *   - ACCESS_COMPOSE_BODYSIZE
+     *     The size of the body data.
+     *
+     *   - ACCESS_COMPOSE_RECIPIENTS
+     *   - ACCESS_COMPOSE_TIMELIMIT
+     *     The number of e-mail recipients.
+     * </pre>
      *
      * @return boolean  Is the access allowed?
      */
-    public function accessCompose($right, $email_count)
+    public function accessCompose($right, $data)
     {
         switch ($right) {
+        case self::ACCESS_COMPOSE_BODYSIZE:
+            $perm_name = 'max_bodysize';
+            break;
+
         case self::ACCESS_COMPOSE_RECIPIENTS:
+            $perm_name = 'max_recipients';
+            break;
+
         case self::ACCESS_COMPOSE_TIMELIMIT:
-            return $GLOBALS['injector']->getInstance('Horde_Core_Perms')->hasAppPermission(
-                ($right == self::ACCESS_COMPOSE_RECIPIENTS) ? 'max_recipients' : 'max_timelimit',
-                array(
-                    'opts' => array(
-                        'value' => $email_count
-                    )
+            $perm_name = 'max_timelimit';
+            break;
+
+        default:
+            return false;
+        }
+
+        return $GLOBALS['injector']->getInstance('Horde_Core_Perms')->hasAppPermission(
+            $perm_name,
+            array(
+                'opts' => array(
+                    'value' => $data
                 )
-            );
-        }
-
-        return false;
-    }
-
-    /**
-     * Get the namespace list.
-     *
-     * @return array  See Horde_Imap_Client_Base#getNamespaces().
-     */
-    public function getNamespaceList()
-    {
-        try {
-            $ns = $this->config->namespace;
-            return $this->getNamespaces(is_null($ns) ? array() : $ns);
-        } catch (Horde_Imap_Client_Exception $e) {
-            return array();
-        }
+            )
+        );
     }
 
     /**
      * Get namespace info for a full mailbox path.
      *
-     * @param string $mailbox    The mailbox path.
+     * @param string $mailbox    The mailbox path. (self:NS_DEFAULT will
+     *                           return the default personal namespace.)
      * @param boolean $personal  If true, will return empty namespace only
      *                           if it is a personal namespace.
      *
      * @return mixed  The namespace info for the mailbox path or null if the
      *                path doesn't exist.
      */
-    public function getNamespace($mailbox = null, $personal = false)
+    public function getNamespace($mailbox, $personal = false)
     {
-        if (!$this->isImap($mailbox)) {
-            return null;
-        }
-
-        $ns = $this->getNamespaceList();
-
-        if (is_null($mailbox)) {
-            reset($ns);
-            $mailbox = key($ns);
-        }
-
-        foreach ($ns as $key => $val) {
-            $mbox = $mailbox . $val['delimiter'];
-            if (strlen($key) && (strpos($mbox, $key) === 0)) {
-                return $val;
+        if ($this->isImap()) {
+            $ns = $this->getNamespaces();
+            if ($mailbox !== self::NS_DEFAULT) {
+                return $ns->getNamespace($mailbox, $personal);
             }
-        }
 
-        return (isset($ns['']) && (!$personal || ($val['type'] == Horde_Imap_Client::NS_PERSONAL)))
-            ? $ns['']
-            : null;
-    }
-
-    /**
-     * Get the default personal namespace.
-     *
-     * @return mixed  The default personal namespace info.
-     */
-    public function defaultNamespace()
-    {
-        if (!$this->_ob ||
-            !$this->isImap() ||
-            !$this->_ob->getParam('imp:login')) {
-            return null;
-        }
-
-        if (is_null($ns = $this->_ob->getParam('imp:nsdefault'))) {
-            foreach ($this->getNamespaceList() as $val) {
-                if ($val['type'] == Horde_Imap_Client::NS_PERSONAL) {
-                    $this->_ob->setParam('imp:nsdefault', $val);
-                    $this->_changed = true;
+            foreach ($ns as $val) {
+                if ($val->type === $val::NS_PERSONAL) {
                     return $val;
                 }
             }
         }
 
-        return $ns;
+        return null;
     }
 
     /**
@@ -488,12 +531,88 @@ class IMP_Imap implements Serializable
         return $out;
     }
 
+    /**
+     * Returns a list of messages, split into slices based on the total
+     * message size.
+     *
+     * @param string $mbox                IMAP mailbox.
+     * @param Horde_Imap_Client_Ids $ids  ID list.
+     * @param integer $size               Maximum size of a slice.
+     *
+     * @return array  An array of Horde_Imap_Client_Ids objects.
+     */
+    public function getSlices(
+        $mbox, Horde_Imap_Client_Ids $ids, $size = 5242880
+    )
+    {
+        $imp_imap = IMP_Mailbox::get($mbox)->imp_imap;
+
+        $query = new Horde_Imap_Client_Fetch_Query();
+        $query->size();
+
+        try {
+            $res = $imp_imap->fetch($mbox, $query, array(
+                'ids' => $ids,
+                'nocache' => true
+            ));
+        } catch (IMP_Imap_Exception $e) {
+            return array();
+        }
+
+        $curr = $slices = array();
+        $curr_size = 0;
+
+        foreach ($res as $key => $val) {
+            $curr_size += $val->getSize();
+            if ($curr_size > $size) {
+                $slices[] = $imp_imap->getIdsOb($curr, $ids->sequence);
+                $curr = array();
+            }
+            $curr[] = $key;
+        }
+
+        $slices[] = $imp_imap->getIdsOb($curr, $ids->sequence);
+
+        return $slices;
+    }
+
+    /**
+     * Handle status() calls. This call may hit multiple servers.
+     *
+     * @see Horde_Imap_Client_Base#status()
+     */
+    protected function _status($args)
+    {
+        global $injector;
+
+        $accounts = $mboxes = $out = array();
+        $imap_factory = $injector->getInstance('IMP_Factory_Imap');
+
+        foreach (IMP_Mailbox::get($args[0]) as $val) {
+            if ($raccount = $val->remote_account) {
+                $accounts[strval($raccount)] = $raccount;
+            }
+            $mboxes[strval($raccount)][] = $val;
+        }
+
+        foreach ($mboxes as $key => $val) {
+            $imap = $imap_factory->create($key);
+            if ($imap->init) {
+                foreach (call_user_func_array(array($imap, 'impStatus'), array($val) + $args) as $key2 => $val2) {
+                    $out[isset($accounts[$key]) ? $accounts[$key]->mailbox($key2) : $key2] = $val2;
+                }
+            }
+        }
+
+        return $out;
+    }
+
     /**
      * All other calls to this class are routed to the underlying
      * Horde_Imap_Client_Base object.
      *
      * @param string $method  Method name.
-     * @param array $params   Method Parameters.
+     * @param array $params   Method parameters.
      *
      * @return mixed  The return from the requested method.
      * @throws BadMethodCallException
@@ -501,7 +620,9 @@ class IMP_Imap implements Serializable
      */
     public function __call($method, $params)
     {
-        if (!$this->_ob) {
+        global $injector;
+
+        if (!$this->init) {
             /* Fallback for these methods. */
             switch ($method) {
             case 'getIdsOb':
@@ -510,11 +631,10 @@ class IMP_Imap implements Serializable
                 return $ob;
             }
 
-            throw new Horde_Exception_AuthenticationFailure('IMP is marked as authenticated, but no credentials can be found in the session.', Horde_Auth::REASON_SESSION);
-        }
-
-        if (!method_exists($this->_ob, $method)) {
-            throw new BadMethodCallException(sprintf('%s: Invalid method call "%s".', __CLASS__, $method));
+            throw new Horde_Exception_AuthenticationFailure(
+                'IMP is marked as authenticated, but no credentials can be found in the session.',
+                Horde_Auth::REASON_SESSION
+            );
         }
 
         switch ($method) {
@@ -531,8 +651,6 @@ class IMP_Imap implements Serializable
         case 'getSyncToken':
         case 'setMetadata':
         case 'setQuota':
-        case 'status':
-        case 'statusMultiple':
         case 'store':
         case 'subscribeMailbox':
         case 'sync':
@@ -544,10 +662,33 @@ class IMP_Imap implements Serializable
 
         case 'copy':
         case 'renameMailbox':
+            // These calls may hit multiple servers.
+            $source = IMP_Mailbox::get($params[0]);
+            $dest = IMP_Mailbox::get($params[1]);
+            if ($source->remote_account != $dest->remote_account) {
+                return call_user_func_array(array($this, '_' . $method), $params);
+            }
+
             // Horde_Imap_Client_Mailbox: these calls all have the mailbox as
             // their first two parameters.
+            $params[0] = $source->imap_mbox_ob;
+            $params[1] = $dest->imap_mbox_ob;
+            break;
+
+        case 'getNamespaces':
+            if (isset($this->_temp['ns'])) {
+                return $this->_temp['ns'];
+            }
+            $nsconfig = $this->config->namespace;
+            $params[0] = is_null($nsconfig) ? array() : $nsconfig;
+            $params[1] = array('ob_return' => true);
+            break;
+
+        case 'impStatus':
+            /* Internal method: allows status call with array of mailboxes,
+             * guaranteeing they are all on this server. */
             $params[0] = IMP_Mailbox::getImapMboxOb($params[0]);
-            $params[1] = IMP_Mailbox::getImapMboxOb($params[1]);
+            $method = 'status';
             break;
 
         case 'openMailbox':
@@ -562,36 +703,76 @@ class IMP_Imap implements Serializable
         case 'search':
             $params = call_user_func_array(array($this, '_search'), $params);
             break;
+
+        case 'status':
+            if (is_array($params[0])) {
+                return $this->_status($params);
+            }
+            $params[0] = IMP_Mailbox::getImapMboxOb($params[0]);
+            break;
+
+        default:
+            if (!method_exists($this->_ob, $method)) {
+                throw new BadMethodCallException(
+                    sprintf('%s: Invalid method call "%s".', __CLASS__, $method)
+                );
+            }
+            break;
         }
 
         try {
             $result = call_user_func_array(array($this->_ob, $method), $params);
         } catch (Horde_Imap_Client_Exception $e) {
-            $error = new IMP_Imap_Exception($e);
-            if ($auth_e = $error->authException(false)) {
-                throw $auth_e;
+            switch ($method) {
+            case 'getNamespaces':
+                return new Horde_Imap_Client_Namespace_List();
             }
 
-            Horde::log($error);
-            throw $error;
+            Horde::log(
+                new Exception(
+                    sprintf('[%s] %s', $method, $e->raw_msg),
+                    $e->getCode(),
+                    $e
+                ),
+                'WARN'
+            );
+            $error = new IMP_Imap_Exception($e);
+            throw ($auth_e = $error->authException(false))
+                ? $auth_e
+                : $error;
         }
 
         /* Special handling for various methods. */
         switch ($method) {
         case 'createMailbox':
+        case 'deleteMailbox':
         case 'renameMailbox':
-            // Mailbox is first parameter.
-            IMP_Mailbox::get($params[0])->expire();
+            $injector->getInstance('IMP_Mailbox_SessionCache')->expire(
+                null,
+                // Mailbox is first parameter.
+                IMP_Mailbox::get($params[0])
+            );
+            break;
+
+        case 'getNamespaces':
+            $this->_temp['ns'] = $result;
             break;
 
         case 'login':
             if (!$this->_ob->getParam('imp:login')) {
                 /* Check for POP3 UIDL support. */
-                if (!$this->isImap() &&
-                    !$this->queryCapability('UIDL')) {
-                    $error = new IMP_Imap_Exception('The POP3 server does not support the REQUIRED UIDL capability.');
-                    Horde::log($error);
-                    throw $error;
+                if ($this->isPop3() && !$this->queryCapability('UIDL')) {
+                    Horde::log(
+                        sprintf(
+                            'The POP3 server does not support the REQUIRED UIDL capability. [server key: %s]',
+                            $this->server_key
+                        ),
+                        'CRIT'
+                    );
+                    throw new Horde_Exception_AuthenticationFailure(
+                        _("The mail server is not currently avaliable."),
+                        Horde_Auth::REASON_MESSAGE
+                    );
                 }
 
                 $this->_ob->setParam('imp:login', true);
@@ -600,7 +781,10 @@ class IMP_Imap implements Serializable
             break;
 
         case 'setACL':
-            IMP_Mailbox::get($params[0])->expire(IMP_Mailbox::CACHE_ACL);
+            $injector->getInstance('IMP_Mailbox_SessionCache')->expire(
+                IMP_Mailbox_SessionCache::CACHE_ACL,
+                IMP_Mailbox::get($params[0])
+            );
             break;
         }
 
@@ -647,6 +831,82 @@ class IMP_Imap implements Serializable
         return array($mailbox->imap_mbox_ob, $query, $opts);
     }
 
+    /**
+     * Handle copy() calls that hit multiple servers.
+     *
+     * @see Horde_Imap_Client_Base#copy()
+     */
+    protected function _copy()
+    {
+        global $injector;
+
+        $args = func_get_args();
+        $imap_factory = $injector->getInstance('IMP_Factory_Imap');
+        $source_imap = $imap_factory->create($args[0]);
+        $dest_imap = $imap_factory->create($args[1]);
+
+        $create = !empty($args[2]['create']);
+        $ids = isset($args[2]['ids'])
+            ? $args[2]['ids']
+            : $source_imap->getIdsOb(Horde_Imap_Client_Ids::ALL);
+        $move = !empty($args[2]['move']);
+        $retval = true;
+
+        $query = new Horde_Imap_Client_Fetch_Query();
+        $query->fullText(array(
+            'peek' => true
+        ));
+
+        foreach ($this->getSlices($args[0], $ids) as $val) {
+            try {
+                $res = $source_imap->fetch($args[0], $query, array(
+                    'ids' => $val,
+                    'nocache' => true
+                ));
+
+                $append = array();
+                foreach ($res as $msg) {
+                    $append[] = array(
+                        'data' => $msg->getFullMsg(true)
+                    );
+                }
+
+                $dest_imap->append($args[1], $append, array(
+                    'create' => $create
+                ));
+
+                if ($move) {
+                    $source_imap->expunge($args[0], array(
+                        'delete' => true,
+                        'ids' => $val
+                    ));
+                }
+            } catch (IMP_Imap_Exception $e) {
+                $retval = false;
+            }
+        }
+
+        return $retval;
+    }
+
+    /**
+     * Handle copy() calls. This call may hit multiple servers, so
+     * need to handle separately from other IMAP calls.
+     *
+     * @see Horde_Imap_Client_Base#renameMailbox()
+     */
+    protected function _renameMailbox()
+    {
+        $args = func_get_args();
+        $source = IMP_Mailbox::get($args[0]);
+
+        if ($source->create() && $this->copy($source, $args[1])) {
+            $source->delete();
+        } else {
+            throw new IMP_Imap_Exception(_("Could not move all messages between mailboxes, so the original mailbox was not removed."));
+        }
+    }
+
     /* Static methods. */
 
     /**
@@ -659,12 +919,11 @@ class IMP_Imap implements Serializable
      */
     static public function loadServerConfig($server = null)
     {
+        global $registry;
+
         if (empty(self::$_backends)) {
             try {
-                $s = Horde::loadConfiguration('backends.php', 'servers', 'imp');
-                if (is_null($s)) {
-                    return false;
-                }
+                $s = $registry->loadConfigFile('backends.php', 'servers', 'imp')->config['servers'];
             } catch (Horde_Exception $e) {
                 Horde::log($e, 'ERR');
                 return false;
@@ -688,10 +947,17 @@ class IMP_Imap implements Serializable
      */
     public function serialize()
     {
-        return serialize(array(
-            $this->_ob,
-            $this->_config
-        ));
+        return $GLOBALS['injector']->getInstance('Horde_Pack')->pack(
+            array(
+                $this->_ob,
+                $this->_id,
+                $this->_config
+            ),
+            array(
+                'compression' => false,
+                'phpob' => true
+            )
+        );
     }
 
     /**
@@ -700,8 +966,9 @@ class IMP_Imap implements Serializable
     {
         list(
             $this->_ob,
+            $this->_id,
             $this->_config
-        ) = unserialize($data);
+        ) = $GLOBALS['injector']->getInstance('Horde_Pack')->unpack($data);
     }
 
 }
