@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2002-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2002-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2002-2013 Horde LLC
+ * @copyright 2002-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -17,14 +17,16 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2002-2013 Horde LLC
+ * @copyright 2002-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
-class IMP_Mailbox_List implements ArrayAccess, Countable, Iterator, Serializable
+class IMP_Mailbox_List
+implements ArrayAccess, Countable, Iterator, Serializable
 {
-    /* Serialized version. */
-    const VERSION = 3;
+    /* The UID count at which the sorted list will undergo UID compression
+     * when being serialized. */
+    const SERIALIZE_LIMIT = 500;
 
     /**
      * Has the internal message list changed?
@@ -173,18 +175,17 @@ class IMP_Mailbox_List implements ArrayAccess, Countable, Iterator, Serializable
             ));
         }
 
-        $imp_imap = $GLOBALS['injector']->getInstance('IMP_Imap');
-
         if (empty($options['preview'])) {
             $cache = null;
             $options['preview'] = 0;
         } else {
-            $cache = $imp_imap->getCache();
+            $cache = $this->_mailbox->imp_imap->getCache();
         }
 
         /* Retrieve information from each mailbox. */
         foreach ($to_process as $mbox => $ids) {
             try {
+                $imp_imap = IMP_Mailbox::get($mbox)->imp_imap;
                 $fetch_res = $imp_imap->fetch($mbox, $fetch_query, array(
                     'ids' => $imp_imap->getIdsOb($ids)
                 ));
@@ -282,7 +283,7 @@ class IMP_Mailbox_List implements ArrayAccess, Countable, Iterator, Serializable
      */
     public function buildMailboxPage($page = 0, $start = 0)
     {
-        global $injector, $prefs, $session;
+        global $prefs, $session;
 
         $this->_buildMailbox();
 
@@ -332,7 +333,7 @@ class IMP_Mailbox_List implements ArrayAccess, Countable, Iterator, Serializable
         $ret['anymsg'] = true;
         if (!$ret['msgcount'] && !$this->_mailbox->search) {
             try {
-                $status = $injector->getInstance('IMP_Imap')->status($this->_mailbox, Horde_Imap_Client::STATUS_MESSAGES);
+                $status = $this->_mailbox->imp_imap->status($this->_mailbox, Horde_Imap_Client::STATUS_MESSAGES);
                 $ret['anymsg'] = (bool)$status['messages'];
             } catch (IMP_Imap_Exception $e) {
                 $ret['anymsg'] = false;
@@ -370,7 +371,6 @@ class IMP_Mailbox_List implements ArrayAccess, Countable, Iterator, Serializable
         $this->_cacheid = $cacheid;
         $this->_sorted = array();
 
-        $imp_imap = $GLOBALS['injector']->getInstance('IMP_Imap');
         $query_ob = $this->_buildMailboxQuery();
         $sortpref = $this->_mailbox->getSort(true);
         $thread_sort = ($sortpref->sortby == Horde_Imap_Client::SORT_THREAD);
@@ -404,7 +404,12 @@ class IMP_Mailbox_List implements ArrayAccess, Countable, Iterator, Serializable
                     $sorted = array_reverse($sorted);
                 }
             } else {
-                $res = $imp_imap->search($mbox, $val, array(
+                $mbox_ob = IMP_Mailbox::get($mbox);
+                if ($mbox_ob->container) {
+                    continue;
+                }
+
+                $res = $mbox_ob->imp_imap->search($mbox, $val, array(
                     'sort' => array($sortpref->sortby)
                 ));
                 if ($sortpref->sortdir) {
@@ -454,14 +459,14 @@ class IMP_Mailbox_List implements ArrayAccess, Countable, Iterator, Serializable
         }
 
         $criteria = new Horde_Imap_Client_Search_Query();
-        $imp_imap = $GLOBALS['injector']->getInstance('IMP_Imap');
+        $imp_imap = $this->_mailbox->imp_imap;
 
         if ($this->_mailbox->hideDeletedMsgs()) {
             $criteria->flag(Horde_Imap_Client::FLAG_DELETED, false);
         } elseif ($count) {
             try {
                 $status_res = $imp_imap->status($this->_mailbox, Horde_Imap_Client::STATUS_UNSEEN);
-                return $status_res[Horde_Imap_Client::STATUS_UNSEEN];
+                return $status_res['unseen'];
             } catch (IMP_Imap_Exception $e) {
                 return 0;
             }
@@ -509,7 +514,7 @@ class IMP_Mailbox_List implements ArrayAccess, Countable, Iterator, Serializable
              * information is returned via a SELECT/EXAMINE call. */
             if ($sortpref->sortby == Horde_Imap_Client::SORT_SEQUENCE) {
                 try {
-                    $res = $GLOBALS['injector']->getInstance('IMP_Imap')->status($this->_mailbox, Horde_Imap_Client::STATUS_FIRSTUNSEEN | Horde_Imap_Client::STATUS_MESSAGES);
+                    $res = $this->_mailbox->imp_imap->status($this->_mailbox, Horde_Imap_Client::STATUS_FIRSTUNSEEN | Horde_Imap_Client::STATUS_MESSAGES);
                     if (!is_null($res['firstunseen'])) {
                         return $sortpref->sortdir
                             ? ($res['messages'] - $res['firstunseen'] + 1)
@@ -714,7 +719,7 @@ class IMP_Mailbox_List implements ArrayAccess, Countable, Iterator, Serializable
     protected function _getThread($mbox, array $extra = array())
     {
         if (!isset($this->_thread[strval($mbox)])) {
-            $imp_imap = $GLOBALS['injector']->getInstance('IMP_Imap');
+            $imp_imap = IMP_Mailbox::get($mbox)->imp_imap;
 
             try {
                 $thread = $imp_imap->thread($mbox, array_merge($extra, array(
@@ -940,7 +945,13 @@ class IMP_Mailbox_List implements ArrayAccess, Countable, Iterator, Serializable
      */
     public function serialize()
     {
-        return serialize($this->_serialize());
+        return $GLOBALS['injector']->getInstance('Horde_Pack')->pack(
+            $this->_serialize(),
+            array(
+                'compression' => false,
+                'phpob' => true
+            )
+        );
     }
 
     /**
@@ -948,8 +959,7 @@ class IMP_Mailbox_List implements ArrayAccess, Countable, Iterator, Serializable
     protected function _serialize()
     {
         $data = array(
-            'm' => $this->_mailbox,
-            'v' => self::VERSION
+            'm' => $this->_mailbox
         );
 
         if ($this->_buidmax) {
@@ -964,7 +974,16 @@ class IMP_Mailbox_List implements ArrayAccess, Countable, Iterator, Serializable
         }
 
         if (!is_null($this->_sorted)) {
-            $data['so'] = $this->_sorted;
+            /* Store UIDs in sequence string to save on storage space. */
+            if (count($this->_sorted) > self::SERIALIZE_LIMIT) {
+                $ids = $this->_mailbox->imp_imap->getIdsOb();
+                /* Optimization: we know there are no duplicates in sorted. */
+                $ids->duplicates = true;
+                $ids->add($this->_sorted);
+                $data['so'] = $ids;
+            } else {
+                $data['so'] = $this->_sorted;
+            }
         }
 
         return $data;
@@ -979,14 +998,9 @@ class IMP_Mailbox_List implements ArrayAccess, Countable, Iterator, Serializable
      */
     public function unserialize($data)
     {
-        $data = @unserialize($data);
-        if (!is_array($data) ||
-            !isset($data['v']) ||
-            ($data['v'] != self::VERSION)) {
-            throw new Exception('Cache version change');
-        }
-
-        $this->_unserialize($data);
+        $this->_unserialize(
+            $GLOBALS['injector']->getInstance('Horde_Pack')->unpack($data)
+        );
     }
 
     /**
@@ -1007,7 +1021,9 @@ class IMP_Mailbox_List implements ArrayAccess, Countable, Iterator, Serializable
         }
 
         if (isset($data['so'])) {
-            $this->_sorted = $data['so'];
+            $this->_sorted = is_object($data['so'])
+                ? $data['so']->ids
+                : $data['so'];
         }
     }
 
