@@ -2,7 +2,7 @@
 /**
  * Message page for minimal view.
  *
- * Copyright 2012-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2012-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
@@ -19,12 +19,11 @@ class IMP_Minimal_Message extends IMP_Minimal_Base
      *   a: (string) Action ID.
      *   allto: (boolean) View all To addresses?
      *   buid: (string) Browser UID.
-     *   mt: (string) Message token.
-     *   fullmsg: (boolean) View full message?
+     *   t: (string) Token.
      */
     protected function _init()
     {
-        global $injector, $notification, $page_output, $prefs;
+        global $injector, $notification, $page_output, $prefs, $session;
 
         $imp_mailbox = $this->indices->mailbox->list_ob;
         $imp_mailbox->setIndex($this->indices);
@@ -47,12 +46,12 @@ class IMP_Minimal_Message extends IMP_Minimal_Base
         case 'd':
             $old_index = $imp_mailbox->getIndex();
             try {
-                $injector->getInstance('Horde_Token')->validate($this->vars->mt, 'imp.message-mimp');
+                $session->checkToken($this->vars->t);
                 $msg_delete = (bool)$injector->getInstance('IMP_Message')->delete(
                     $this->indices,
                     array('mailboxob' => $imp_mailbox)
                 );
-            } catch (Horde_Token_Exception $e) {
+            } catch (Horde_Exception $e) {
                 $notification->push($e);
             }
             break;
@@ -92,7 +91,7 @@ class IMP_Minimal_Message extends IMP_Minimal_Base
 
         /* Get envelope/flag/header information. */
         try {
-            $imp_imap = $injector->getInstance('IMP_Imap');
+            $imp_imap = $mailbox->imp_imap;
 
             /* Need to fetch flags before HEADERTEXT, because SEEN flag might
              * be set before we can grab it. */
@@ -205,19 +204,11 @@ class IMP_Minimal_Message extends IMP_Minimal_Base
         /* Create the body of the message. */
         $inlineout = $imp_contents->getInlineOutput(array(
             'display_mask' => IMP_Contents::RENDER_INLINE,
-            'no_inline_all' => !$prefs->getValue('mimp_inline_all')
+            'no_inline_all' => true
         ));
 
         $msg_text = $inlineout['msgtext'];
 
-        /* Display the first 250 characters, or display the entire message? */
-        if ($prefs->getValue('mimp_preview_msg') &&
-            !isset($this->vars->fullmsg) &&
-            (strlen($msg_text) > 250)) {
-            $msg_text = Horde_String::substr(trim($msg_text), 0, 250) . " [...]\n";
-            $this->view->fullmsg_link = $self_link->copy()->add('fullmsg', 1);
-        }
-
         $this->view->msg = nl2br($injector->getInstance('Horde_Core_Factory_TextFilter')->filter($msg_text, 'space2html'));
 
         $compose_params = array(
@@ -230,7 +221,7 @@ class IMP_Minimal_Message extends IMP_Minimal_Base
         if ($this->indices->mailbox->access_deletemsgs) {
             $menu[] = in_array(Horde_Imap_Client::FLAG_DELETED, $flags)
                 ? array(_("Undelete"), $self_link->copy()->add('a', 'u'))
-                : array(_("Delete"), $self_link->copy()->add(array('a' => 'd', 'mt' => $injector->getInstance('Horde_Token')->get('imp.message-mimp'))));
+                : array(_("Delete"), $self_link->copy()->add(array('a' => 'd', 't' => $session->getToken())));
         }
 
         /* Add compose actions (Reply, Reply List, Reply All, Forward,
@@ -275,11 +266,11 @@ class IMP_Minimal_Message extends IMP_Minimal_Base
         $menu[] = array(sprintf(_("To %s"), $this->indices->mailbox->label), $mailbox_link);
 
         if ($mailbox->spam_show) {
-            $menu[] = array(_("Report as Spam"), $self_link->copy()->add(array('a' => 'rs', 'mt' => $injector->getInstance('Horde_Token')->get('imp.message-mimp'))));
+            $menu[] = array(_("Report as Spam"), $self_link->copy()->add(array('a' => 'rs', 't' => $session->getToken())));
         }
 
         if ($mailbox->innocent_show) {
-            $menu[] = array(_("Report as Innocent"), $self_link->copy()->add(array('a' => 'ri', 'mt' => $injector->getInstance('Horde_Token')->get('imp.message-mimp'))));
+            $menu[] = array(_("Report as Innocent"), $self_link->copy()->add(array('a' => 'ri', 't' => $session->getToken())));
         }
 
         $this->view->menu = $this->getMenu('message', $menu);
@@ -313,12 +304,10 @@ class IMP_Minimal_Message extends IMP_Minimal_Base
             if (!empty($summary['download'])) {
                 /* Preference: if set, only show download confirmation screen
                  * if attachment over a certain size. */
-                $tmp['download'] = ($summary['bytes'] > $prefs->getValue('mimp_download_confirm'))
-                    ? IMP_Minimal_Messagepart::url(array(
-                          'buid' => $buid,
-                          'mailbox' => $this->indices->mailbox
-                      ))->add('atc', $key)
-                    : $summary['download_url'];
+                $tmp['download'] = IMP_Minimal_Messagepart::url(array(
+                    'buid' => $buid,
+                    'mailbox' => $this->indices->mailbox
+                ))->add('atc', $key);
             }
 
             if ($imp_contents->canDisplay($key, IMP_Contents::RENDER_INLINE)) {
