@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2012-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2012-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
  *
  * @category  Horde
- * @copyright 2012-2013 Horde LLC
+ * @copyright 2012-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -17,7 +17,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2012-2013 Horde LLC
+ * @copyright 2012-2014 Horde LLC
  * @license   http://www.horde.org/licenses/gpl GPL
  * @package   IMP
  */
@@ -36,6 +36,10 @@ class IMP_Script_Package_ComposeBase extends Horde_Script_Package
             switch ($registry->getView()) {
             case $registry::VIEW_DYNAMIC:
                 $this->_files[] = new Horde_Script_File_JsDir('ckeditor/imageupload.js', 'imp');
+                $this->_files[] = new Horde_Script_File_JsDir('ckeditor/imagepoll.js', 'imp');
+                $page_output->addInlineJsVars(array(
+                    'IMP_Ckeditor_Imagepoll.related_attr' => IMP_Compose::RELATED_ATTR
+                ));
 
                 $plugin = 'pasteattachment';
                 $upload_url = $registry->getServiceLink('ajax', 'imp')->url . 'addAttachmentCkeditor';
@@ -51,9 +55,7 @@ class IMP_Script_Package_ComposeBase extends Horde_Script_Package
             $page_output->addInlineScript(array(
                 'if (window.CKEDITOR) { CKEDITOR.on("loaded", function(e) {' .
                   'CKEDITOR.plugins.addExternal("' . $plugin . '", "' . $js->url->url . '", "");' .
-                  'CKEDITOR.config.extraPlugins = CKEDITOR.config.extraPlugins.split(",").concat("' . $plugin . '").join(",");' .
                   'CKEDITOR.config.filebrowserImageUploadUrl = "' . $upload_url . '";' .
-                  'CKEDITOR.config.forcePasteAsPlainText = true;' .
                 '}); };'
             ), true);
         }
